@@ -130,7 +130,7 @@ instanceof Instanceof  @op
 isa? Isa                @op
 isa  TT_ISA @alias
 as  As                 @op
-in? In                 @op
+in? Has                 @op
 #in  TT_IN             @alias
 === Is                 @op @2
 is?  TT_IS @alias
