@@ -1289,36 +1289,28 @@ void knh_Stmt_terms_cmpl_(Ctx *ctx, Stmt *stmt, int n, Compiler *cpr, NameSpace
 			return;
 		}
 
-		if(reqc == CLASS_Boolean) {
-			Object *v = knh_Term_constValue(ctx, tm, CLASS_Any);
-			if(IS_bInt(v)) {
-				DEBUG3_ASSERT(IS_Token(DP(stmt)->tokens[n]));
-				Token *tkc = DP(stmt)->tokens[n];
-				knh_Token_perror(ctx, tkc, KMSG_WTRUEFALSE);
-				if(((Int*)v)->value == 0) {
-					knh_perrata(ctx, SP(tkc)->fileid, SP(tkc)->line, "0", "false");
-					knh_Token_setCONST(ctx, DP(stmt)->tokens[n], KNH_FALSE);
-				}
-				else {
-					char bufn[80];
-					knh_snprintf(bufn, sizeof(bufn), "%d", (int)((Int*)v)->value);
-					knh_perrata(ctx, SP(tkc)->fileid, SP(tkc)->line, bufn, "true");
-					knh_Token_setCONST(ctx, DP(stmt)->tokens[n], KNH_TRUE);
-				}
-			}
-			else if(IS_NULL(v) && knh_Term_isCONST(DP(stmt)->terms[n])) {
-				knh_Token_setCONST(ctx, DP(stmt)->tokens[n], KNH_FALSE);
-			}
-			else if(!IS_Boolean(v) && IS_Token(tm)) {
-				DEBUG3_ASSERT(IS_Token(DP(stmt)->tokens[n]));
-				knh_Token_perror(ctx, DP(stmt)->tokens[n], KMSG_WTRUEFALSE);
-				knh_Token_setCONST(ctx, DP(stmt)->tokens[n], KNH_TRUE);
-			}
-			if(IS_Token(tm) && push) {
-				KNH_ASM_PPUSH(ctx, cpr, tm, level);
-			}
-			return ;
-		}
+//		if(reqc == CLASS_Boolean) {
+//			Object *v = knh_Term_constValue(ctx, tm, CLASS_Any);
+//			if(IS_bInt(v)) {
+//				DEBUG3_ASSERT(IS_Token(DP(stmt)->tokens[n]));
+//				Token *tkc = DP(stmt)->tokens[n];
+//				knh_Token_perror(ctx, tkc, KMSG_WTRUEFALSE);
+//				if(((Int*)v)->value == 0) {
+//					knh_perrata(ctx, SP(tkc)->fileid, SP(tkc)->line, "0", "false");
+//					knh_Token_setCONST(ctx, DP(stmt)->tokens[n], KNH_FALSE);
+//				}
+//				else {
+//					char bufn[80];
+//					knh_snprintf(bufn, sizeof(bufn), "%d", (int)((Int*)v)->value);
+//					knh_perrata(ctx, SP(tkc)->fileid, SP(tkc)->line, bufn, "true");
+//					knh_Token_setCONST(ctx, DP(stmt)->tokens[n], KNH_TRUE);
+//				}
+//			}
+//			if(IS_Token(tm) && push) {
+//				KNH_ASM_PPUSH(ctx, cpr, tm, level);
+//			}
+//			return ;
+//		}
 
 		if(varc == CLASS_Nue) {
 			if(TYPE_ISNOTNULL(reqt)) {
