@@ -108,7 +108,7 @@ void KNH_ASM_LABEL__b(Ctx *ctx, Compiler *cpr, knh_bytes_t t)
 		return;
 	}
 	knhvmc_t *pc = (knhvmc_t*)knh_Bytes_last(DP(cpr)->elf);
-	knh_DictSet_put(ctx, DP(cpr)->labelAddrDictSet, new_String(ctx, t, NULL), (knh_uint_t)pc);
+	knh_DictSet_set(ctx, DP(cpr)->labelAddrDictSet, new_String(ctx, t, NULL), (knh_uint_t)pc);
 	//DEBUG("SET LABEL '%s' at %p", t.buf, pc);
 }
 
