@@ -277,12 +277,6 @@ void knh_Token_constCMETHODN(Ctx *ctx, Token *o, NameSpace *ns)
 	}
 	t = knh_bytes_last(t, idx+1);
 	knh_methodn_t mn = knh_tName_getMethodn(ctx, t, METHODN_NONAME);
-	if(mn == METHODN_NONAME) {
-		knh_Token_perror(ctx, o, KMSG_UMETHODN);
-		knh_Token_perrata(ctx, o, "null");
-		knh_Token_setCONST(ctx, o, KNH_NULL);
-		return;
-	}
 	Method *mtd = knh_Class_getMethod(ctx, cid, mn);
 	if(IS_NULL(mtd)) {
 		knh_Token_perror(ctx, o, KMSG_UMETHODN);
@@ -722,7 +716,7 @@ Token *knh_Token_const(Ctx *ctx, Token *o, knh_class_t reqc, NameSpace *ns)
 					knh_Token_setCONST(ctx, o, KNH_FALSE);
 				}
 				else if(IS_Int(value)) {
-					knh_Token_perror(ctx, o, KMSG_WTRUEFALSE);
+					knh_perror(ctx, SP(o)->fileid, SP(o)->line, KMSG_WTRUEFALSE, NULL);
 					if(((Int*)value)->value == 0) {
 						knh_perrata(ctx, SP(o)->fileid, SP(o)->line, "0", "false");
 						knh_Token_setCONST(ctx, o, KNH_FALSE);
