@@ -66,7 +66,7 @@ Stmt *new_StmtDECL(Ctx *ctx, Token *tk_TYPEN, knh_tokens_t *tc, int start)
 	L_NEXT:;
 	//DEBUG3("N tc->c=%d, tc->e=%d comma.c=%d comma.e=%d", tc->c, tc->e, comma_tc.c, comma_tc.e);
 	if(tc->c < tc->e) {
-		if(SP(tc->ts[tc->c])->tt == TT_SEMICOLON || knh_token_isEndOfStmt(SP(tc->ts[tc->c])->tt)) {
+		if(/* SP(tc->ts[tc->c])->tt == TT_SEMICOLON || */knh_Token_isBOL(tc->ts[tc->c])) {
 			return stmt_b;
 		}
 		else {
@@ -220,7 +220,6 @@ static Token *knh_tokens_findLETNULL(knh_tokens_t *tc)
 	Token **ts = tc->ts;
 	int i;
 	for(i = tc->c; i < tc->e; i++) {
-		KNH_ASSERT(!knh_token_isEndOfStmt(SP(ts[i])->tt));
 		if(TT_LET <= SP(ts[i])->tt && SP(ts[i])->tt <= TT_ALTLET) return ts[i];
 	}
 	return NULL;
@@ -277,10 +276,7 @@ Stmt* new_StmtLETEXPR(Ctx *ctx, knh_tokens_t *tc)
 		}
 	}
 	else {
-		//DEBUG3_STC("S expr_tc", expr_tc);
 		Term *term = new_TermEXPR(ctx, &expr_tc, KNH_STT_RVALUE);
-		//DEBUG3_STC("E expr_tc", expr_tc);
-		KNH_ASSERT(term != NULL);
 		if(IS_Token(term)) {
 			Stmt *stmt = new_Stmt(ctx, 0, STT_CALL1);
 			knh_Stmt_add(ctx, stmt, term);
@@ -579,7 +575,7 @@ Token *knh_tokens_findIDXNULL(knh_tokens_t *tc)
 	int i;
 	for(i = tc->c; i < tc->e; i++) {
 		knh_token_t tt = SP(tc->ts[i])->tt;
-		KNH_ASSERT(!knh_token_isEndOfStmt(tt));
+		KNH_ASSERT(!knh_Token_isBOL(tc->ts[i]));
 		if(tt == TT_SUBSETE || tt == TT_OFFSET || tt == TT_SUBSET) {
 			return tc->ts[i];
 		}
@@ -955,8 +951,8 @@ Stmt* new_StmtFUNCEXPR(Ctx *ctx, knh_tokens_t *tc)
 Stmt *new_StmtINSTMT(Ctx *ctx, Token *tk)
 {
 	knh_tokens_t tc;
-	Stmt *stmt_b = NULL;
-#if defined(KONOHA_SAFEMODE)
+	Stmt *first_stmt = NULL;
+#if defined(KNH_DBGMODE2)
 	int prev = 0;
 #endif
 
@@ -964,32 +960,32 @@ Stmt *new_StmtINSTMT(Ctx *ctx, Token *tk)
 	knh_Token_tc(tk, &tc);
 
 	while(tc.c < tc.e) {
-		//DEBUG("S s = %d, c = %d, e = %d", (int)s, (int)c, (int)e);
+		//DBG2_P("S c = %d, e = %d", (int)tc.c, (int)tc.e);
 		if(SP(tc.ts[tc.c])->tt == TT_SEMICOLON) {
-			tc.c++;
+			tc.c += 1;
 			continue;
 		}
-#if defined(KONOHA_SAFEMODE)
+#if defined(KNH_DBGMODE2)
 		prev = tc.c;
 #endif
-		if(stmt_b == NULL) {
-			Stmt *stmt_b2 = new_StmtSTMT1(ctx, &tc);
-			if(stmt_b2 != NULL) stmt_b = stmt_b2;
+		if(first_stmt == NULL) {
+			Stmt *stmt = new_StmtSTMT1(ctx, &tc);
+			if(stmt != NULL) first_stmt = stmt;
 		}else{
-			knh_Stmt_tail_append(ctx, stmt_b, new_StmtSTMT1(ctx, &tc));
+			knh_Stmt_tail_append(ctx, first_stmt, new_StmtSTMT1(ctx, &tc));
 		}
-#if defined(KONOHA_SAFEMODE)
+#if defined(KNH_DBGMODE2)
 		if(prev == tc.c) { /* infinate loop */
-			DEBUG("Infinate loop? prev = %d, c = %d, e = %d", prev, tc.c, tc.e);
+			DBG2_P("Infinate loop? prev = %d, c = %d, e = %d", prev, tc.c, tc.e);
 			break;
 		}
 #endif
 	}
-	if(stmt_b == NULL) {
-		stmt_b = new_StmtDONE(ctx);
+	if(first_stmt == NULL) {
+		first_stmt = new_StmtDONE(ctx);
 	}
-	//DEBUG("E s = %d, c = %d, e = %d", (int)s, (int)c, (int)e);
-	return stmt_b;
+	//DBG2_P("E c = %d, e = %d", (int)tc.c, (int)tc.e);
+	return first_stmt;
 }
 
 /* ======================================================================== */
@@ -1000,7 +996,7 @@ void knh_tokens_checkLET(Ctx *ctx, knh_tokens_t *tc)
 {
 	int i;
 	for(i = tc->c; i < tc->e; i++) {
-		KNH_ASSERT(!knh_token_isEndOfStmt(SP(tc->ts[i])->tt));
+		KNH_ASSERT(!knh_Token_isBOL(tc->ts[i]));
 		if(SP(tc->ts[i])->tt == TT_LET) {
 			knh_Token_perror(ctx, tc->ts[i], KMSG_WLET);
 			knh_Token_perrata(ctx, tc->ts[i], "==");
@@ -1018,7 +1014,7 @@ Term *new_TermPEXPR(Ctx *ctx, Token *tk)
 		knh_tokens_t p_tc;
 		knh_Token_tc(tk, &p_tc);
 		if(p_tc.e == 0) {
-			knh_perror(ctx, SP(tk)->fileid, SP(tk)->line, KMSG_WEMPTYPEXPR, "()");
+			knh_perror(ctx, SP(tk)->fileid, SP(tk)->line, KMSG_WEMPTY, "()");
 			knh_perrata(ctx, SP(tk)->fileid, SP(tk)->line, "()", "(false)");
 			return TM(new_TokenCONST(ctx, FL(tk), KNH_FALSE));
 		}
@@ -1236,6 +1232,7 @@ Term *new_TermEXPR(Ctx *ctx, knh_tokens_t *tc, int lr)
 	if(!(oc < e)) {
 		DBG2_P("tc->c=%d, tc->e=%d", (int)oc, (int)e);
 		KNH_ASSERT(e > 0);
+		KNH_ASSERT(oc < e);
 		Token *tke = new_Token(ctx, 0, SP(tc->ts[e-1])->fileid, SP(tc->ts[e-1])->line, TT_ERR);
 		knh_perror(ctx, SP(tc->ts[e-1])->fileid, SP(tc->ts[e-1])->line, KMSG_ESYNTAX, NULL);
 		return TM(tke);
@@ -1404,7 +1401,6 @@ Term *new_TermEXPR(Ctx *ctx, knh_tokens_t *tc, int lr)
 	KNH_ASSERT(stmt != NULL);
 	KNH_ASSERT(DP(stmt)->size == 1);
 	KNH_ASSERT(fc > 0);
-	DEBUG3("Funcname %s", knh_token_tochar(SP(ts[fc])->tt));
 
 	if(SP(ts[fc])->tt == TT_NAME && knh_Token_isTopDot(ts[fc])) {
 		stmt = knh_Stmt_add_SECONDNAME(ctx, stmt, ts[fc], lr);
@@ -1419,19 +1415,21 @@ Term *new_TermEXPR(Ctx *ctx, knh_tokens_t *tc, int lr)
 			goto L_FUNC;
 		}
 		else {
-			tc->c = fc + 1;
-			knh_tokens_ignore(ctx, tc);
+			goto L_IGNORE_FUNCNAME;
 		}
 		return TM(stmt);
 	}
-	if(SP(ts[fc])->tt == TT_BRANCET) {
+	else if(SP(ts[fc])->tt == TT_BRANCET) {
 		knh_Stmt_add_IDX(ctx, stmt, ts[fc], lr);
 		fc = fc + 1;
 		goto L_NEXTFUNC;
 	}
 	else {
-		tc->c = fc;
-		knh_tokens_ignore(ctx, tc);
+		L_IGNORE_FUNCNAME:
+		DEBUG3("** Unexpected funcname %s **", knh_token_tochar(SP(ts[fc])->tt));
+		knh_Token_perror(ctx, ts[fc], KMSG_ESYNTAX);
+		knh_Stmt_add(ctx, stmt, TM(ts[fc]));
+		tc->c = tc->e;
 	}
 	return TM(stmt);
 
@@ -1447,7 +1445,7 @@ Term *new_TermEXPR(Ctx *ctx, knh_tokens_t *tc, int lr)
 		knh_Stmt_add_EXPRs(ctx, stmt, &ptc);
 	}
 	if(DP(stmt)->size == 1) {
-		knh_perror(ctx, SP(ts[pc])->fileid, SP(ts[pc])->line, KMSG_WEMPTYARGS, "()");
+		knh_perror(ctx, SP(ts[pc])->fileid, SP(ts[pc])->line, KMSG_WEMPTY, "()");
 		knh_perrata(ctx, SP(ts[pc])->fileid, SP(ts[pc])->line, "()", "(null)");
 		knh_Stmt_add(ctx, stmt, TM(new_TokenCONST(ctx, FL(ts[oc]), KNH_NULL)));
 	}
@@ -1468,7 +1466,8 @@ void knh_Stmt_add_EXPRs(Ctx *ctx, Stmt *stmt_b, knh_tokens_t *tc)
 {
 	while(tc->c < tc->e) {
 		knh_tokens_t sub_tc;
-		if(SP(tc->ts[tc->c])->tt == TT_SEMICOLON || knh_token_isEndOfStmt(SP(tc->ts[tc->c])->tt)) {
+		Token *tk = tc->ts[tc->c];
+		if(/* SP(tk)->tt == TT_SEMICOLON || */ knh_Token_isBOL(tk)) {
 			return ;
 		}
 		//DEBUG3_PTC("tc", tc);
@@ -1479,7 +1478,6 @@ void knh_Stmt_add_EXPRs(Ctx *ctx, Stmt *stmt_b, knh_tokens_t *tc)
 	}
 }
 
-
 /* ------------------------------------------------------------------------ */
 
 #ifdef __cplusplus
