@@ -35,37 +35,47 @@
 extern "C" {
 #endif
 
+
+#undef DEBUG3
+#define DEBUG3(fmt, ...)
+
 /* ======================================================================== */
 /* [data] */
 
 #define PSTACK_MAXSIZ 32
 
 typedef struct {  /* @data */
-	struct knh_Token_t *tk;
-	int indent;
-} pstack_t;
+	struct knh_Token_t *tk;  /* parent */
+	int indent;  /*parent */
+	int previndent;
+	int curindent;
+	int tkcount;
+} knh_tokenctx_t;
 
 static knh_bool_t knh_Bytes_isTripleQuote(Bytes *b, knh_int_t ch);
-static pstack_t *knh_pstack_back(Ctx *ctx, pstack_t *ps, pstack_t *psp, int indent);
+static knh_tokenctx_t *knh_pstack_back(Ctx *ctx, knh_tokenctx_t *ps, knh_tokenctx_t *psp, int indent);
 static Token *new_Token__buffer(Ctx *ctx, knh_token_t tt, knh_wbuf_t tbuf, InputStream *in);
-static void knh_Token_padd(Ctx *ctx, Token *b, Token *tk);
+static void knh_Token_padd(Ctx *ctx, knh_tokenctx_t *tkctx, Token *tk);
+
 #define knh_Token_add_whitespace  knh_Token_add_space
-static void knh_Token_add_space(Ctx *ctx, Token *b, knh_wbuf_t tbuf, InputStream *in);
+static void knh_Token_add_space(Ctx *ctx, knh_tokenctx_t *tkctx, knh_wbuf_t tbuf, InputStream *in, int join);
 
 static int knh_InputStream_getc__urlencoded(Ctx *ctx, InputStream *in);
 
 /* ======================================================================== */
 /* [parse] */
 
-void knh_Token_parse(Ctx *ctx, Token *tk, InputStream *in)
+void knh_Token_parse(Ctx *ctx, Token *tk, InputStream *in, int join)
 {
 	knh_wbuf_t tbuf = knh_Context_wbuf(ctx);
 	int ch, prev=0, indent = 0;
-	pstack_t ps[PSTACK_MAXSIZ], *psp = ps;
-	psp->tk = tk;
-	psp->indent = 0;
+	knh_tokenctx_t tkctx_stack[PSTACK_MAXSIZ], *tkctx = tkctx_stack;
+	tkctx->tk = tk;
+	tkctx->indent = 0;
 
 	INDENT_PART:;
+	tkctx->previndent = indent;
+	tkctx->tkcount = 0;
 	indent = 0;
 	while((ch = knh_InputStream_getc(ctx, in)) != EOF) {
 		if(ch == '\t') {
@@ -75,14 +85,19 @@ void knh_Token_parse(Ctx *ctx, Token *tk, InputStream *in)
 			indent++;
 		}
 		else if(ch == '/' || ch == '\n' || ch == '\r' || ch == EOF) {
+			tkctx->curindent = indent;
 			goto MAIN_PART_INLOOP;
 		}
-		break;
+		else {
+			break;
+		}
 	}
-	if(indent < psp->indent) {
-		DEBUG3("deindent %d -> %d", indent, psp->indent);
-		psp = knh_pstack_back(ctx, ps, psp, indent);
+
+	if(indent < tkctx->indent) {
+		knh_perror(ctx, DP(in)->fileid, DP(in)->line, KMSG_WINDENT, NULL);
+		tkctx = knh_pstack_back(ctx, tkctx_stack, tkctx, indent);
 	}
+	tkctx->curindent = indent;
 	goto MAIN_PART_INLOOP;
 
 	MAIN_PART:;
@@ -91,29 +106,32 @@ void knh_Token_parse(Ctx *ctx, Token *tk, InputStream *in)
 
 		switch(ch) {
 		case '\n':
-			knh_Token_add_whitespace(ctx, psp->tk, tbuf, in);
+			knh_Token_add_whitespace(ctx, tkctx, tbuf, in, join);
+
 		goto INDENT_PART;
 
 		case '\t': case ' ': case '\v': case '\r':
-			knh_Token_add_whitespace(ctx, psp->tk, tbuf, in);
+			knh_Token_add_whitespace(ctx, tkctx, tbuf, in, join);
 			prev = ch;
+
 		goto MAIN_PART;
 
 		case '{':
-			knh_Token_add_space(ctx, psp->tk, tbuf, in);
-			if(SP(psp->tk)->tt == TT_PARENTHESIS && IS_NOTNULL(DP(psp->tk)->data)) {
-				DBG2_P("@Probabry forget ')': indent=%d, psp->indent=%d", indent, psp->indent);
+			knh_Token_add_space(ctx, tkctx, tbuf, in, join);
+			if(SP(tkctx->tk)->tt == TT_PARENTHESIS && IS_NOTNULL(DP(tkctx->tk)->data)) {
+				DBG2_P("@Probabry forget ')': indent=%d, psp->indent=%d", indent, tkctx->indent);
 				knh_perror(ctx, DP(in)->fileid, DP(in)->line, KMSG_EFORGET_PARENTHESIS, "..) {");
-				psp = knh_pstack_back(ctx, ps, psp, indent);
+				tkctx = knh_pstack_back(ctx, tkctx_stack, tkctx, indent);
 			}
+
 		case '(': 	case '[':
-			knh_Token_add_space(ctx, psp->tk, tbuf, in);
-			if(psp+1 < ps+PSTACK_MAXSIZ) {
+			knh_Token_add_space(ctx, tkctx, tbuf, in, join);
+			if(tkctx+1 < tkctx_stack+PSTACK_MAXSIZ) {
 				Token *intk = new_Token(ctx, 0, DP(in)->fileid, DP(in)->line, knh_char_totoken(ch));
-				knh_Token_padd(ctx, psp->tk, intk);
-				psp++;
-				psp->tk = intk;
-				psp->indent = indent;
+				knh_Token_padd(ctx, tkctx, intk);
+				tkctx++;
+				tkctx->tk = intk;
+				tkctx->indent = indent;
 			}
 			else {
 				DBG2_P("pstack overflow!!");
@@ -122,13 +140,13 @@ void knh_Token_parse(Ctx *ctx, Token *tk, InputStream *in)
 			goto MAIN_PART;
 
 		case '}':
-			knh_Token_add_space(ctx, psp->tk, tbuf, in);
-			if(SP(psp->tk)->tt == TT_BRACE) {
-				knh_Token_retokens(ctx, psp->tk);
-				psp--;
-				if(psp < ps) {
+			knh_Token_add_space(ctx, tkctx, tbuf, in, join);
+			if(SP(tkctx->tk)->tt == TT_BRACE) {
+				knh_Token_retokens(ctx, tkctx->tk);
+				tkctx--;
+				if(tkctx < tkctx_stack) {
 					knh_perror(ctx, DP(in)->fileid, DP(in)->line, KMSG_EMISMATCH, "{ ...}");
-					psp++;
+					tkctx++;
 				}
 			}
 			else {
@@ -137,11 +155,11 @@ void knh_Token_parse(Ctx *ctx, Token *tk, InputStream *in)
 			goto MAIN_PART;
 
 		case ')':
-			knh_Token_add_space(ctx, psp->tk, tbuf, in);
-			if(SP(psp->tk)->tt == TT_PARENTHESIS) {
-				knh_Token_retokens(ctx, psp->tk);
-				psp--;
-				KNH_ASSERT(ps <= psp);
+			knh_Token_add_space(ctx, tkctx, tbuf, in, join);
+			if(SP(tkctx->tk)->tt == TT_PARENTHESIS) {
+				knh_Token_retokens(ctx, tkctx->tk);
+				tkctx--;
+				KNH_ASSERT(tkctx_stack <= tkctx);
 			}
 			else {
 				knh_perror(ctx, DP(in)->fileid, DP(in)->line, KMSG_EMISMATCH, "( ...)");
@@ -149,11 +167,11 @@ void knh_Token_parse(Ctx *ctx, Token *tk, InputStream *in)
 			goto MAIN_PART;
 
 		case ']':
-			knh_Token_add_space(ctx, psp->tk, tbuf, in);
-			if(SP(psp->tk)->tt == TT_BRANCET) {
-				knh_Token_retokens(ctx, psp->tk);
-				psp--;
-				KNH_ASSERT(ps <= psp);
+			knh_Token_add_space(ctx, tkctx, tbuf, in, join);
+			if(SP(tkctx->tk)->tt == TT_BRANCET) {
+				knh_Token_retokens(ctx, tkctx->tk);
+				tkctx--;
+				KNH_ASSERT(tkctx_stack <= tkctx);
 			}
 			else {
 				knh_perror(ctx, DP(in)->fileid, DP(in)->line, KMSG_EMISMATCH, "[ ...]");
@@ -161,22 +179,22 @@ void knh_Token_parse(Ctx *ctx, Token *tk, InputStream *in)
 			goto MAIN_PART;
 
 		case '"': case '\'': case '`' :
-			knh_Token_add_space(ctx, psp->tk, tbuf, in);
+			knh_Token_add_space(ctx, tkctx, tbuf, in, join);
 			prev = ch;
 			goto QUOTED_PART;
 
 		case ',': case ';':
-			knh_Token_add_space(ctx, psp->tk, tbuf, in);
-			knh_Token_padd(ctx, psp->tk, new_Token(ctx, 0, DP(in)->fileid, DP(in)->line, knh_char_totoken(ch)));
+			knh_Token_add_space(ctx, tkctx, tbuf, in, join);
+			knh_Token_padd(ctx, tkctx, new_Token(ctx, 0, DP(in)->fileid, DP(in)->line, knh_char_totoken(ch)));
 			break;
 
 		case '$':
-			knh_Token_add_space(ctx, psp->tk, tbuf, in);
+			knh_Token_add_space(ctx, tkctx, tbuf, in, join);
 			prev = ch;
 			goto QNAME_PART;
 
 		case '@':
-			knh_Token_add_space(ctx, psp->tk, tbuf, in);
+			knh_Token_add_space(ctx, tkctx, tbuf, in, join);
 			ch = knh_InputStream_getc(ctx, in);
 			if(ch == '"' || ch == '\'' || ch == '`') {
 				prev = ch;
@@ -187,7 +205,7 @@ void knh_Token_parse(Ctx *ctx, Token *tk, InputStream *in)
 			goto QNAME_PART;
 
 		case '%':
-			knh_Token_add_space(ctx, psp->tk, tbuf, in);
+			knh_Token_add_space(ctx, tkctx, tbuf, in, join);
 			ch = knh_InputStream_getc(ctx, in);
 			if(isalnum(ch)) {
 				prev = '%';
@@ -197,38 +215,38 @@ void knh_Token_parse(Ctx *ctx, Token *tk, InputStream *in)
 			else if(ch == '%') {
 				knh_Bytes_putc(ctx, tbuf.ba, '%');
 				knh_Bytes_putc(ctx, tbuf.ba, '%');
-				knh_Token_add_space(ctx, psp->tk, tbuf, in);
+				knh_Token_add_space(ctx, tkctx, tbuf, in, join);
 				break;
 			}
 			else {
 				knh_Bytes_putc(ctx, tbuf.ba, '%');
-				knh_Token_add_space(ctx, psp->tk, tbuf, in);
+				knh_Token_add_space(ctx, tkctx, tbuf, in, join);
 				goto MAIN_PART_INLOOP;
 			}
 
 		case '+': case '-': case '*': case '=':
 		case '&': case '|': case '<':
 		case '>': case '^': case '!': case '~':
-			knh_Token_add_space(ctx, psp->tk, tbuf, in);
+			knh_Token_add_space(ctx, tkctx, tbuf, in, join);
 			knh_Bytes_putc(ctx, tbuf.ba, ch);
 			prev = ch;
 			goto OP_PART;
 
 		case '?':
 			if(!islower(prev)) {
-				knh_Token_add_space(ctx, psp->tk, tbuf, in);
+				knh_Token_add_space(ctx, tkctx, tbuf, in, join);
 				knh_Bytes_putc(ctx, tbuf.ba, ch);
 				prev = ch;
 				goto OP_PART;
 			}
 			else {
 				knh_Bytes_putc(ctx, tbuf.ba, ch);
-				knh_Token_add_space(ctx, psp->tk, tbuf, in);
+				knh_Token_add_space(ctx, tkctx, tbuf, in, join);
 				break;
 			}
 
 		case '/':
-			knh_Token_add_space(ctx, psp->tk, tbuf, in);
+			knh_Token_add_space(ctx, tkctx, tbuf, in, join);
 			ch = knh_InputStream_getc(ctx, in);
 			if(ch == '*') {
 				goto BLOCK_COMMENT;
@@ -240,13 +258,13 @@ void knh_Token_parse(Ctx *ctx, Token *tk, InputStream *in)
 			goto OP_PART_INLOOP;
 
 		case '#':
-			knh_Token_add_space(ctx, psp->tk, tbuf, in);
+			knh_Token_add_space(ctx, tkctx, tbuf, in, join);
 			goto LINE_COMMENT;
 
 		case '.':
 			ch = knh_InputStream_getc(ctx, in);
 			if(ch == '.') { /* .. */
-				knh_Token_add_space(ctx, psp->tk, tbuf, in);
+				knh_Token_add_space(ctx, tkctx, tbuf, in, join);
 				knh_Bytes_putc(ctx, tbuf.ba, '.');
 				knh_Bytes_putc(ctx, tbuf.ba, ch);
 				prev = '.';
@@ -255,7 +273,7 @@ void knh_Token_parse(Ctx *ctx, Token *tk, InputStream *in)
 			else if(ch == '*') {
 				knh_Bytes_putc(ctx, tbuf.ba, '.');
 				knh_Bytes_putc(ctx, tbuf.ba, '*');
-				knh_Token_add_space(ctx, psp->tk, tbuf, in);
+				knh_Token_add_space(ctx, tkctx, tbuf, in, join);
 				prev = ' ';
 				break;
 			}
@@ -270,7 +288,7 @@ void knh_Token_parse(Ctx *ctx, Token *tk, InputStream *in)
 			if(knh_wbuf_size(tbuf) == 0) {
 				if(ch == ' ' || ch == '\n' || ch == '\t' || ch == '\r' || ch == ';') {
 					knh_Bytes_putc(ctx, tbuf.ba, ':');
-					knh_Token_add_space(ctx, psp->tk, tbuf, in);
+					knh_Token_add_space(ctx, tkctx, tbuf, in, join);
 					break;
 				}
 				knh_Bytes_putc(ctx, tbuf.ba, ':');
@@ -282,7 +300,7 @@ void knh_Token_parse(Ctx *ctx, Token *tk, InputStream *in)
 				goto URN_PART;
 			}
 			else if(ch == ' ' || ch == '\n' || ch == '\t' || ch == '\r' || ch == ';') {
-				knh_Token_padd(ctx, psp->tk, new_Token__buffer(ctx, TT_LABEL, tbuf, in));
+				knh_Token_padd(ctx, tkctx, new_Token__buffer(ctx, TT_LABEL, tbuf, in));
 				//knh_Bytes_putc(ctx, tbuf.ba, ':');
 				goto MAIN_PART_INLOOP;
 			}
@@ -313,7 +331,7 @@ void knh_Token_parse(Ctx *ctx, Token *tk, InputStream *in)
 		} /* switch */
 		prev = ch;
 	}/* while */
-	knh_Token_add_space(ctx, psp->tk, tbuf, in);
+	knh_Token_add_space(ctx, tkctx, tbuf, in, join);
 	return ;
 
 	NUM_PART:;  /* 0x0000 */
@@ -331,7 +349,7 @@ void knh_Token_parse(Ctx *ctx, Token *tk, InputStream *in)
 				case '.':
 					ch = knh_InputStream_getc(ctx, in);
 					if(ch == '.') {
-						knh_Token_padd(ctx, psp->tk, new_Token__buffer(ctx, TT_NUM, tbuf, in));
+						knh_Token_padd(ctx, tkctx, new_Token__buffer(ctx, TT_NUM, tbuf, in));
 						knh_Bytes_putc(ctx, tbuf.ba, '.');
 						goto OP_PART_INLOOP;
 					}
@@ -349,12 +367,12 @@ void knh_Token_parse(Ctx *ctx, Token *tk, InputStream *in)
 				case ']':   /* 1.0[km/h] */
 					unit--;
 					if(unit < 0) {
-						knh_Token_padd(ctx, psp->tk, new_Token__buffer(ctx, TT_NUM, tbuf, in));
+						knh_Token_padd(ctx, tkctx, new_Token__buffer(ctx, TT_NUM, tbuf, in));
 						goto MAIN_PART_INLOOP;
 					}
 					else if(unit == 0) {
 						knh_Bytes_putc(ctx, tbuf.ba, ch);
-						knh_Token_padd(ctx, psp->tk, new_Token__buffer(ctx, TT_NUM, tbuf, in));
+						knh_Token_padd(ctx, tkctx, new_Token__buffer(ctx, TT_NUM, tbuf, in));
 						goto MAIN_PART;
 					}
 				break;
@@ -364,13 +382,13 @@ void knh_Token_parse(Ctx *ctx, Token *tk, InputStream *in)
 						knh_Bytes_putc(ctx, tbuf.ba, ch);
 					}
 					else {
-						knh_Token_padd(ctx, psp->tk, new_Token__buffer(ctx, TT_NUM, tbuf, in));
+						knh_Token_padd(ctx, tkctx, new_Token__buffer(ctx, TT_NUM, tbuf, in));
 						goto MAIN_PART_INLOOP;
 					}
 				/*break*/
 			}
 		}
-		knh_Token_padd(ctx, psp->tk, new_Token__buffer(ctx, TT_NUM, tbuf, in));
+		knh_Token_padd(ctx, tkctx, new_Token__buffer(ctx, TT_NUM, tbuf, in));
 		//knh_perror(ctx, DP(in)->fileid, DP(in)->line, KMSG_WEOF, NULL);
 	}
 	goto MAIN_PART_INLOOP; /* EOF */
@@ -389,7 +407,7 @@ void knh_Token_parse(Ctx *ctx, Token *tk, InputStream *in)
 			}
 			break;
 		}
-		knh_Token_padd(ctx, psp->tk, new_Token__buffer(ctx, knh_char_totoken(prev), tbuf, in));
+		knh_Token_padd(ctx, tkctx, new_Token__buffer(ctx, knh_char_totoken(prev), tbuf, in));
 	}
 	goto MAIN_PART_INLOOP; /* EOF */
 
@@ -402,7 +420,7 @@ void knh_Token_parse(Ctx *ctx, Token *tk, InputStream *in)
 				knh_Bytes_putc(ctx, tbuf.ba, ch);
 			}
 			else {
-				knh_Token_add_space(ctx, psp->tk, tbuf, in);
+				knh_Token_add_space(ctx, tkctx, tbuf, in, join);
 				goto MAIN_PART_INLOOP;
 			}
 //
@@ -439,13 +457,13 @@ void knh_Token_parse(Ctx *ctx, Token *tk, InputStream *in)
 				case '(': case ')':
 				case '{': case '}':
 				case '[': case ']':
-					knh_Token_padd(ctx, psp->tk, new_Token__buffer(ctx, TT_URN, tbuf, in));
+					knh_Token_padd(ctx, tkctx, new_Token__buffer(ctx, TT_URN, tbuf, in));
 					goto MAIN_PART_INLOOP;
 				case '%':
 					ch = knh_InputStream_getc__urlencoded(ctx, in);
 					if(ch == EOF) {
 						ch = '\n';
-						knh_Token_padd(ctx, psp->tk, new_Token__buffer(ctx, TT_URN, tbuf, in));
+						knh_Token_padd(ctx, tkctx, new_Token__buffer(ctx, TT_URN, tbuf, in));
 						goto MAIN_PART_INLOOP;
 					}
 					knh_Bytes_putc(ctx, tbuf.ba, ch);
@@ -453,7 +471,7 @@ void knh_Token_parse(Ctx *ctx, Token *tk, InputStream *in)
 					knh_Bytes_putc(ctx, tbuf.ba, ch);
 			}
 		}
-		knh_Token_padd(ctx, psp->tk, new_Token__buffer(ctx, TT_URN, tbuf, in));
+		knh_Token_padd(ctx, tkctx, new_Token__buffer(ctx, TT_URN, tbuf, in));
 	}/*URN_PART*/
 	return;
 
@@ -463,7 +481,7 @@ void knh_Token_parse(Ctx *ctx, Token *tk, InputStream *in)
 		knh_Bytes_putc(ctx, tbuf.ba, prev);
 		while((ch = knh_InputStream_getc(ctx, in)) != EOF) {
 			if(ch == prev) {
-				knh_Token_padd(ctx, psp->tk, new_Token__buffer(ctx, TT_STR, tbuf, in));
+				knh_Token_padd(ctx, tkctx, new_Token__buffer(ctx, TT_STR, tbuf, in));
 				goto MAIN_PART;
 			}
 			knh_Bytes_putc(ctx, tbuf.ba, ch);
@@ -472,7 +490,7 @@ void knh_Token_parse(Ctx *ctx, Token *tk, InputStream *in)
 			char buf4[4];
 			knh_snprintf(buf4, sizeof(buf4), "%c", prev);
 			knh_perror(ctx, DP(in)->fileid, DP(in)->line, KMSG_WUNCLOSED, buf4);
-			knh_Token_padd(ctx, psp->tk, new_Token__buffer(ctx, TT_STR, tbuf, in));
+			knh_Token_padd(ctx, tkctx, new_Token__buffer(ctx, TT_STR, tbuf, in));
 			goto MAIN_PART_INLOOP;
 		}
 	}/*RAWSTR_PART*/
@@ -488,7 +506,7 @@ void knh_Token_parse(Ctx *ctx, Token *tk, InputStream *in)
 			if(prev == ch) { /* triple quote */
 				goto TRIPLE_QUOTED_PART;
 			}else {
-				knh_Token_padd(ctx, psp->tk, new_Token__buffer(ctx, TT_STR, tbuf, in));
+				knh_Token_padd(ctx, tkctx, new_Token__buffer(ctx, TT_STR, tbuf, in));
 				goto MAIN_PART_INLOOP;
 			}
 		}
@@ -521,11 +539,11 @@ void knh_Token_parse(Ctx *ctx, Token *tk, InputStream *in)
 				char buf4[4];
 				knh_snprintf(buf4, sizeof(buf4), "%c", prev);
 				knh_perror(ctx, DP(in)->fileid, DP(in)->line, KMSG_WUNCLOSED, buf4);
-				knh_Token_padd(ctx, psp->tk, new_Token__buffer(ctx, TT_STR, tbuf, in));
+				knh_Token_padd(ctx, tkctx, new_Token__buffer(ctx, TT_STR, tbuf, in));
 				goto MAIN_PART_INLOOP;
 			}
 			if(ch == prev) {
-				knh_Token_padd(ctx, psp->tk, new_Token__buffer(ctx, TT_STR, tbuf, in));
+				knh_Token_padd(ctx, tkctx, new_Token__buffer(ctx, TT_STR, tbuf, in));
 				goto MAIN_PART;
 			}
 			knh_Bytes_putc(ctx, tbuf.ba, ch);
@@ -568,7 +586,7 @@ void knh_Token_parse(Ctx *ctx, Token *tk, InputStream *in)
 				knh_Bytes_unputc(tbuf.ba);
 				knh_Bytes_unputc(tbuf.ba);
 				knh_Bytes_unputc(tbuf.ba);
-				knh_Token_padd(ctx, psp->tk, new_Token__buffer(ctx, TT_STR, tbuf, in));
+				knh_Token_padd(ctx, tkctx, new_Token__buffer(ctx, TT_STR, tbuf, in));
 				goto MAIN_PART;
 			}
 		}
@@ -576,7 +594,7 @@ void knh_Token_parse(Ctx *ctx, Token *tk, InputStream *in)
 			char buf4[4];
 			knh_snprintf(buf4, sizeof(buf4), "%c%c%c", prev, prev, prev);
 			knh_perror(ctx, DP(in)->fileid, DP(in)->line, KMSG_WUNCLOSED, buf4);
-			knh_Token_padd(ctx, psp->tk, new_Token__buffer(ctx, TT_STR, tbuf, in));
+			knh_Token_padd(ctx, tkctx, new_Token__buffer(ctx, TT_STR, tbuf, in));
 		}
 	}
 	return;
@@ -614,12 +632,15 @@ void knh_Token_parse(Ctx *ctx, Token *tk, InputStream *in)
 
 String *new_String__NAME(Ctx *ctx, knh_bytes_t tname)
 {
-	knh_index_t idx = knh_DictIdx_index(ctx, DP(knh_rootSystem)->tfieldnDictIdx, tname);
-	if(idx != -1) {
-		return knh_DictIdx_get__fast(DP(knh_rootSystem)->tfieldnDictIdx, (knh_fieldn_t)idx);
+	knh_index_t idx = knh_DictMap_index__b((ctx)->tsymbolDictMap, tname);
+	if(idx == -1) {
+		String *s = new_String(ctx, tname, NULL);
+		knh_DictMap_set(ctx, (ctx)->tsymbolDictMap, s, UP(s));
+		return s;
+	}
+	else {
+		return knh_DictMap_keyAt((ctx)->tsymbolDictMap, idx);
 	}
-	DEBUG3("new_String__NAME('%s')", (char*)tname.buf);
-	return new_String(ctx, tname, NULL);
 }
 
 /* ------------------------------------------------------------------------ */
@@ -799,7 +820,10 @@ Token *new_Token__name(Ctx *ctx, knh_flag_t flag, knh_fileid_t fileid, knh_line_
 	}
 	{
 		Token *o = new_Token(ctx, flag, fileid, line, tt);
-		DEBUG3("(%s) '%s' ==> '%s'", knh_token_tochar(tt), (char*)t.buf, name);
+		DBG2_(
+		if(!knh_bytes_equals(t, B(name))) {
+			DBG2_P("(%s) '%s' ==> '%s'", knh_token_tochar(tt), (char*)t.buf, name);
+		});
 		KNH_SETv(ctx, DP(o)->data, new_String__NAME(ctx, B(name)));
 		return o;
 	}
@@ -807,7 +831,7 @@ Token *new_Token__name(Ctx *ctx, knh_flag_t flag, knh_fileid_t fileid, knh_line_
 
 /* ------------------------------------------------------------------------ */
 static
-pstack_t *knh_pstack_back(Ctx *ctx, pstack_t *ps, pstack_t *psp, int indent)
+knh_tokenctx_t *knh_pstack_back(Ctx *ctx, knh_tokenctx_t *ps, knh_tokenctx_t *psp, int indent)
 {
 	DEBUG3("@S indent=%d", indent);
 	while(ps < psp) {
@@ -891,13 +915,25 @@ Token *knh_Token_tokens_lastNULL(Token *o)
 /* ------------------------------------------------------------------------ */
 
 static
-void knh_Token_padd(Ctx *ctx, Token *o, Token *tk)
+void knh_Token_padd(Ctx *ctx, knh_tokenctx_t *tkctx, Token *tk)
 {
-	Token *tkp = knh_Token_tokens_lastNULL(o);
+	Token *tkp = knh_Token_tokens_lastNULL(tkctx->tk);
 	if(tkp != NULL) {
 		DP(tkp)->tt_next = SP(tk)->tt;
+		if(SP(tkctx->tk)->tt == TT_BRACE) {
+			if(tkctx->tkcount == 0 && SP(tk)->line > SP(tkp)->line) {
+				DEBUG3("@@@ curindent=%d, previndent=%d", tkctx->curindent, tkctx->previndent);
+				if(tkctx->previndent == tkctx->curindent) {
+					knh_Token_setBOL(tk, 1);
+				}
+			}
+		}
 	}
-	knh_Token_tokens_add(ctx, o, tk);
+	if(SP(tk)->tt == TT_SEMICOLON || knh_token_isEndOfStmt(SP(tk)->tt)) {
+		knh_Token_setBOL(tk, 1);
+	}
+	knh_Token_tokens_add(ctx, tkctx->tk, tk);
+	tkctx->tkcount += 1;
 }
 
 /* ------------------------------------------------------------------------ */
@@ -943,21 +979,15 @@ void knh_Token_retokens(Ctx *ctx, Token *o)
 #define knh_Token_add_whitespace  knh_Token_add_space
 
 static
-void knh_Token_add_space(Ctx *ctx, Token *o, knh_wbuf_t tbuf, InputStream *in)
+void knh_Token_add_space(Ctx *ctx, knh_tokenctx_t *tkctx, knh_wbuf_t tbuf, InputStream *in, int join)
 {
 	knh_bytes_t t = knh_wbuf_tobytes(tbuf);
-	//DEBUG3("t.len=%d, '%s'", (int)t.len, t.buf);
 
 	if(t.len == 0) {
 		return;
 	}
-	else {
-//		if(t.buf[0] == '.' && t.len == 1) {
-//			TODO3(". Token");
-//			knh_wbuf_clear(tbuf);
-//			return;
-//		}
-		Token *tkp = knh_Token_tokens_lastNULL(o);
+	else if(join) {
+		Token *tkp = knh_Token_tokens_lastNULL(tkctx->tk);
 		if(tkp != NULL) {
 			if(SP(tkp)->tt == TT_TYPEN) {
 				if(t.buf[0] == '.' && t.buf[1] == '.' && t.buf[2] == '\0') {
@@ -977,9 +1007,9 @@ void knh_Token_add_space(Ctx *ctx, Token *o, knh_wbuf_t tbuf, InputStream *in)
 				}
 			}
 		}
-		knh_Token_padd(ctx, o, new_Token__parse(ctx, 0, DP(in)->fileid, DP(in)->line, t, DP(in)->bconv));
-		knh_wbuf_clear(tbuf);
 	}
+	knh_Token_padd(ctx, tkctx, new_Token__parse(ctx, 0, DP(in)->fileid, DP(in)->line, t, DP(in)->bconv));
+	knh_wbuf_clear(tbuf);
 }
 
 /* ------------------------------------------------------------------------ */
