@@ -218,57 +218,72 @@ typedef struct knh_Iterator {
 } knh_Iterator_struct;
 
 /* ------------------------------------------------------------------------ */
-/* @class DictMap2 Object knh_DictMap2_t @Cyclic */
-/* @class DictSet2 Object knh_DictSet2_t @Cyclic */
+/* @class DictMap Object knh_DictMap_t @Cyclic @Param1 */
+/* @class DictSet Object knh_DictSet_t @Cyclic @Param1 */
 
-typedef struct {
+typedef struct knh_dict_t {
+	struct knh_String_t *key;
 	union {
-		Object *key;
-		struct knh_String_t *name;
+		Object *_unused;
+		knh_uint_t _unused_int;
 	};
-	union {
-		Object *value;
-		knh_uint_t uvalue;
-	};
-} knh_dictentry_t;
-
-typedef struct knh_DictMap2_t {
-	knh_hObject_t h;
-	struct dictentry *list;
-	size_t size;
-} knh_DictMap2_t;
-
-#define knh_DictSet2_t    knh_DictMap2_t
-
-/* ------------------------------------------------------------------------ */
-/* @class DictMap Object knh_DictMap_struct */
+} knh_dict_t;
 
 typedef struct knh_dictmape_t {
 	struct knh_String_t *key;
 	Object *value;
 } knh_dictmape_t;
 
-typedef struct knh_DictMap {
+typedef struct knh_DictMap_t {
+	knh_hObject_t h;
+	union {
+		knh_dictmape_t *list;
+		knh_dict_t *_list;  // To avoid stupid casting
+	};
 	size_t size;
-	size_t capacity;
-	knh_dictmape_t* list;
-	size_t sorted;
-} knh_DictMap_struct;
+} knh_DictMap_t;
 
 /* ------------------------------------------------------------------------ */
-/* @class DictSet Object knh_DictSet_struct */
 
 typedef struct knh_dictsete_t {
 	struct knh_String_t *key;
-	knh_uint_t  value;
+	knh_uint_t value;
 } knh_dictsete_t;
 
-typedef struct knh_DictSet {
+typedef struct knh_DictSet_t {
+	knh_hObject_t h;
+	union {
+		knh_dictsete_t *list;
+		knh_dict_t *_list;  // To avoid stupid casting
+	};
 	size_t size;
-	size_t capacity;
-	knh_dictsete_t* list;
-	size_t sorted;
-} knh_DictSet_struct;
+} knh_DictSet_t;
+
+///* ------------------------------------------------------------------------ */
+///* @ class DictMap Object knh_DictMap_struct */
+//
+//
+//typedef struct knh_DictMap {
+//	size_t size;
+//	size_t capacity;
+//	knh_dictmape_t* list;
+//	size_t sorted;
+//} knh_DictMap_struct;
+//
+///* ------------------------------------------------------------------------ */
+///* @ class DictSet Object knh_DictSet_struct */
+//
+//typedef struct knh_dictsete_t {
+//	struct knh_String_t *key;
+//	knh_uint_t  value;
+//} knh_dictsete_t;
+//
+//typedef struct knh_DictSet {
+//	size_t size;
+//	size_t capacity;
+//	knh_dictsete_t* list;
+//	size_t sorted;
+//} knh_DictSet_struct;
 
 /* ------------------------------------------------------------------------ */
 /* @class HashMap Object knh_HashMap_struct @Param2 */
