@@ -101,20 +101,6 @@ Object** knh_oarray_grow(Ctx *ctx, Object **a);
 Object **knh_oarray_copy(Ctx *ctx, Object **a, size_t s, size_t offset);
 void knh_oarray_traverse(Ctx *ctx, Object **a, f_traverse ftr);
 void knh_oarray_free(Ctx *ctx, knh_Object_t **a);
-size_t knh_array_capacity(void *a);
-size_t knh_array_opsize(void *a);
-void knh_array_set_opsize(void *a, knh_short_t opsize);
-int knh_array_op1(void *a);
-void knh_array_set_op1(void *a, knh_short_t op1);
-int knh_array_op2(void *a);
-void knh_array_set_op2(void *a, knh_short_t op2);
-void *knh_array_malloc(Ctx *ctx, size_t capacity, size_t sizeofeach, void (*farrayinit)(Ctx *, void *), void (*farraytraverse)(Ctx*, void*, f_traverse));
-void *knh_array_resize(Ctx *ctx, void *a, size_t newsize);
-void knh_array_clear(Ctx *ctx, void *a);
-void *knh_array_traverse(Ctx *ctx, void *a, f_traverse ftr);
-void* knh_array_fcompare(void *a);
-void knh_array_set_fcompare(void *a, int (*farraycompar)(const void *, const void *));
-void knh_array_sort(void *a);
 /* ../class/konoha/knh_Boolean_api1.c */
 String* knh_Boolean_String(Ctx *ctx, Boolean *o, Mapper *mpr);
 Boolean* knh_String_Boolean(Ctx *ctx, String *o, Mapper *mpr);
@@ -262,51 +248,79 @@ String* knh_DictIdx_get__fast(DictIdx *o, knh_int_t termid);
 String* knh_DictIdx_get(DictIdx *o, knh_int_t termid);
 void knh_DictIdx_clear(Ctx *ctx, DictIdx *o);
 void knh_DictIdx__k(Ctx *ctx, DictIdx *o, OutputStream *w, String *m);
-/* ../class/konoha/knh_DictMap.c */
-void knh_DictMap_struct_init(Ctx *ctx, knh_DictMap_struct *b, int init, Object *cs);
-void knh_DictMap_struct_copy(Ctx *ctx, knh_DictMap_struct *b, knh_DictMap_struct *b2);
-void knh_DictMap_struct_traverse(Ctx *ctx, knh_DictMap_struct *b, f_traverse gc);
-DictMap* new_DictMap(Ctx *ctx, knh_int_t capacity);
-DictMap *knh_DictMap_new(Ctx *ctx, DictMap *o, knh_int_t initialCapacity);
+/* ../class/konoha/knh_DictMap_api1.c */
+METHOD knh__DictMap_new(Ctx *ctx, knh_sfp_t *sfp);
+METHOD knh__DictMap_new__init(Ctx *ctx, knh_sfp_t *sfp);
+METHOD knh__DictMap_get(Ctx *ctx, knh_sfp_t *sfp);
+METHOD knh__DictMap_opHas(Ctx *ctx, knh_sfp_t *sfp);
+METHOD knh__DictMap_set(Ctx *ctx, knh_sfp_t *sfp);
+METHOD knh__DictMap_remove(Ctx *ctx, knh_sfp_t *sfp);
+METHOD knh__DictMap_clear(Ctx *ctx, knh_sfp_t *sfp);
+METHOD knh__DictMap_getSize(Ctx *ctx, knh_sfp_t *sfp);
+METHOD knh__DictMap_subset(Ctx *ctx, knh_sfp_t *sfp, int mode);
+METHOD knh__DictMap_opSubsete(Ctx *ctx, knh_sfp_t *sfp);
+METHOD knh__DictMap_opSubset(Ctx *ctx, knh_sfp_t *sfp);
+METHOD knh__DictMap_opOffset(Ctx *ctx, knh_sfp_t *sfp);
+METHOD knh__DictMap_opLshift(Ctx *ctx, knh_sfp_t *sfp);
+Object *knh_DictMap_key_next(Ctx *ctx, Iterator *it);
+Iterator* knh_DictMap_Iterator(Ctx *ctx, DictMap *o, Mapper *mpr);
+Iterator* knh_DictMap_String__(Ctx *ctx, DictMap *o, Mapper *mpr);
+METHOD knh__DictMap_opItr(Ctx *ctx, knh_sfp_t *sfp);
+void knh_DictMap__k(Ctx *ctx, DictMap *o, OutputStream *w, String *m);
+/* ../class/konoha/knh_DictMapSet.c */
+knh_dict_t* knh_dict_resize(Ctx *ctx, knh_dict_t *a, size_t newsize);
+void knh_dict_traverse(Ctx *ctx, knh_dict_t *a, f_traverse ftr);
 void knh_DictMap_sort(DictMap *o);
-knh_index_t knh_DictMap_index__b(DictMap *o, knh_bytes_t kv);
+void knh_DictSet_sort(DictSet *o);
+knh_dict_t *knh_dictmap_malloc(Ctx *ctx, size_t init);
+DictMap* new_DictMap0(Ctx *ctx, knh_int_t capacity);
+DictMap* new_DictMap(Ctx *ctx, knh_class_t p1, knh_int_t capacity);
 size_t knh_DictMap_size(DictMap *o);
 String* knh_DictMap_keyAt(DictMap *o, size_t n);
 Object* knh_DictMap_valueAt(DictMap *o, size_t n);
 void knh_DictMap_removeAt(Ctx *ctx, DictMap *o, size_t n);
+knh_index_t knh_DictMap_index__b(DictMap *o, knh_bytes_t key);
 String *new_String__DictMap(Ctx *ctx, DictMap *o, knh_bytes_t key);
-Object *knh_DictMap_get__b(Ctx *ctx, DictMap *o, knh_bytes_t key);
+Object *knh_DictMap_get__b(DictMap *o, knh_bytes_t key);
 Object *knh_DictMap_get(Ctx *ctx, DictMap *o, String *key);
-void knh_DictMap_resize(Ctx *ctx, DictMap *o, size_t newsize);
 void knh_DictMap_append(Ctx *ctx, DictMap *o, String *key, Any *value);
 void knh_DictMap_set(Ctx *ctx, DictMap *o, String *key, Any *v);
 void knh_DictMap_remove(Ctx *ctx, DictMap *o, String *key);
 void knh_DictMap_clear(Ctx *ctx, DictMap *o);
-Object *knh_DictMap_key_next(Ctx *ctx, Iterator *it);
-Iterator* knh_DictMap_Iterator(Ctx *ctx, DictMap *o, Mapper *map);
-void knh_DictMap__dump(Ctx *ctx, DictMap *o, OutputStream *w, String *m);
-/* ../class/konoha/knh_DictSet.c */
-void knh_DictSet_struct_init(Ctx *ctx, knh_DictSet_struct *b, int init, Object *cs);
-void knh_DictSet_struct_copy(Ctx *ctx, knh_DictSet_struct *b, knh_DictSet_struct *b2);
-void knh_DictSet_struct_traverse(Ctx *ctx, knh_DictSet_struct *b, f_traverse gc);
-DictSet *new_DictSet(Ctx *ctx, knh_int_t capacity);
-DictSet* knh_DictSet_new(Ctx *ctx, DictSet *b, knh_int_t initialCapacity);
-void knh_DictSet_sort(DictSet *b);
-knh_index_t knh_DictSet_index__b(DictSet *b, knh_bytes_t kv);
-String* knh_DictSet_key(DictSet *b, size_t n);
-knh_uint_t knh_DictSet_value(DictSet *b, size_t n);
+knh_dict_t *knh_dictset_malloc(Ctx *ctx, size_t init);
+DictSet* new_DictSet0(Ctx *ctx, knh_int_t capacity);
+size_t knh_DictSet_size(DictSet *o);
+String* knh_DictSet_keyAt(DictSet *o, size_t n);
+knh_uint_t knh_DictSet_valueAt(DictSet *o, size_t n);
+void knh_DictSet_removeAt(Ctx *ctx, DictSet *o, size_t n);
 String *new_String__DictSet(Ctx *ctx, DictSet *o, knh_bytes_t key);
-knh_uint_t knh_DictSet_get__b(DictSet *b, knh_bytes_t kv);
-knh_int_t knh_DictSet_get(DictSet *b, String *key);
-knh_bool_t knh_DictSet_opHas(Ctx *ctx, DictSet *b, Any *value);
-void knh_DictSet_resize(Ctx *ctx, DictSet *b, size_t newsize);
-void knh_DictSet_append(Ctx *ctx, DictSet *b, String *key, knh_uint_t value);
-void knh_DictSet_set(Ctx *ctx, DictSet *b, String *key, knh_uint_t value);
-void knh_DictSet_add(Ctx *ctx, DictSet *b, String *key);
-void knh_DictSet_clear(Ctx *ctx, DictSet *b);
+knh_uint_t knh_DictSet_get__b(DictSet *o, knh_bytes_t key);
+knh_uint_t knh_DictSet_get(Ctx *ctx, DictSet *o, String *key);
+void knh_DictSet_append(Ctx *ctx, DictSet *o, String *key, knh_uint_t n);
+void knh_DictSet_set(Ctx *ctx, DictSet *o, String *key, knh_uint_t n);
+void knh_DictSet_add(Ctx *ctx, DictSet *o, String *key);
+void knh_DictSet_remove(Ctx *ctx, DictSet *o, String *key);
+void knh_DictSet_clear(Ctx *ctx, DictSet *o);
 /* ../class/konoha/knh_DictSet_api1.c */
-void knh_DictSet__dump(Ctx *ctx, DictSet *b, OutputStream *w, String *m);
+METHOD knh__DictSet_new(Ctx *ctx, knh_sfp_t *sfp);
+METHOD knh__DictSet_add(Ctx *ctx, knh_sfp_t *sfp);
+METHOD knh__DictSet_get(Ctx *ctx, knh_sfp_t *sfp);
+METHOD knh__DictSet_opHas(Ctx *ctx, knh_sfp_t *sfp);
+METHOD knh__DictSet_set(Ctx *ctx, knh_sfp_t *sfp);
+METHOD knh__DictSet_remove(Ctx *ctx, knh_sfp_t *sfp);
+METHOD knh__DictSet_clear(Ctx *ctx, knh_sfp_t *sfp);
+METHOD knh__DictSet_getSize(Ctx *ctx, knh_sfp_t *sfp);
+METHOD knh__DictSet_subset(Ctx *ctx, knh_sfp_t *sfp, int mode);
+METHOD knh__DictSet_opSubsete(Ctx *ctx, knh_sfp_t *sfp);
+METHOD knh__DictSet_opSubset(Ctx *ctx, knh_sfp_t *sfp);
+METHOD knh__DictSet_opOffset(Ctx *ctx, knh_sfp_t *sfp);
+METHOD knh__DictSet_opLshift(Ctx *ctx, knh_sfp_t *sfp);
 Iterator* knh_DictSet_Iterator(Ctx *ctx, DictSet *o, Mapper *mpr);
+Iterator* knh_DictSet_String__(Ctx *ctx, DictSet *o, Mapper *mpr);
+METHOD knh__DictSet_opItr(Ctx *ctx, knh_sfp_t *sfp);
+DictSet* knh_String___DictSet(Ctx *ctx, Iterator *o, Mapper *mpr);
+void knh_DictSet__k(Ctx *ctx, DictSet *o, OutputStream *w, String *m);
+void knh_DictSet__dump(Ctx *ctx, DictSet *o, OutputStream *w, String *m);
 /* ../class/konoha/knh_Exception.c */
 void knh_Exception_struct_init(Ctx *ctx, knh_Exception_struct *b, int init, Object *cs);
 void knh_Exception_struct_traverse(Ctx *ctx, knh_Exception_struct *b, f_traverse gc);
@@ -701,12 +715,6 @@ void knh_Runtime_setAliasURN(Ctx *ctx, String *alias, String *urn);
 Script *knh_NameSpace_getScript(Ctx *ctx, NameSpace *ns);
 void knh_Script__k(Ctx *ctx, Script *o, OutputStream *w, String *m);
 void knh_Script__dump(Ctx *ctx, Script *o, OutputStream *w, String *m);
-/* ../class/konoha/knh_Socket_utils.c */
-char * knh_bytes_urlencode(knh_bytes_t s, char *buf, size_t bufsiz);
-char * knh_bytes_urldecode(knh_bytes_t s, char *buf, size_t bufsiz);
-char * knh_bytes_urlhost(knh_bytes_t url, char *buf, size_t bufsiz);
-int knh_bytes_urlport(knh_bytes_t url, int defport);
-char * knh_bytes_urlpath(knh_bytes_t url, char *buf, size_t bufsiz);
 /* ../class/konoha/knh_String.c */
 knh_hcode_t knh_string_hash(knh_uchar_t *buf, knh_size_t len);
 void knh_String_checkASCII(String *o);
@@ -852,7 +860,7 @@ void knh_Stmt_add_PARAMs(Ctx *ctx, Stmt *o, knh_tokens_t *tc);
 Stmt *new_StmtMETHOD(Ctx *ctx, knh_tokens_t *tc);
 void knh_Stmt_add_PEACH(Ctx *ctx, Stmt *o, knh_tokens_t *tc);
 /* ../class/konohac/knh_parser_Token.c */
-void knh_Token_parse(Ctx *ctx, Token *tk, InputStream *in);
+void knh_Token_parse(Ctx *ctx, Token *tk, InputStream *in, int join);
 String *new_String__NAME(Ctx *ctx, knh_bytes_t tname);
 Token *new_Token__name(Ctx *ctx, knh_flag_t flag, knh_fileid_t fileid, knh_line_t line, knh_bytes_t t);
 void knh_Token_retokens(Ctx *ctx, Token *o);
@@ -866,7 +874,6 @@ void knh_tokens_perror(Ctx *ctx, knh_tokens_t *tc, int pe);
 void knh_Stmt_perror(Ctx *ctx, Stmt *b, knh_tokens_t *tc, int pe);
 Stmt *new_StmtERR(Ctx *ctx, knh_tokens_t *tc);
 void knh_Compiler_perror(Ctx *ctx, Compiler *cpr, int pe, char *msg);
-void knh_notice_TypeInf(Ctx *ctx, knh_fileid_t fileid, int line, char *msg, knh_type_t type);
 /* ../class/konohac/knh_Stmt.c */
 void knh_Stmt_struct_init(Ctx *ctx, knh_Stmt_struct *b, int init, Object *cs);
 void knh_Stmt_struct_traverse(Ctx *ctx, knh_Stmt_struct *b, f_traverse gc);
