@@ -77,7 +77,7 @@
 /* ======================================================================== */
 /* [WINDOWS] */
 
-#ifdef KONOHA_OS__TB
+#ifdef KONOHA_OS__TBMX
 #define 	KNH_USING_BTRON    1
 #define 	KONOHA_OS_ENCODING KONOHA_ENCODING
 #define 	KONOHA_OS_LINEFEED "\n"
