@@ -81,6 +81,7 @@ static int knh_message_typedata[] = {
     /* KMSG_WUNCLOSED */ 2,
     /* KMSG_WASCII */ 2,
     /* KMSG_WEOF */ 2,
+    /* KMSG_WINDENT */ 2,
     /* KMSG_WSEMICOLON */ 2,
     /* KMSG_IGSTMT */ 2,
     /* KMSG_ETOKEN */ 1,
@@ -122,8 +123,7 @@ static int knh_message_typedata[] = {
     /* KMSG_EPSTMT3 */ 1,
     /* KMSG_EPEACH */ 1,
     /* KMSG_WLET */ 2,
-    /* KMSG_WEMPTYPEXPR */ 2,
-    /* KMSG_WEMPTYARGS */ 2,
+    /* KMSG_WEMPTY */ 2,
     /* KMSG_WTRUEFALSE */ 2,
     /* KMSG_USCRIPT */ 1,
     /* KMSG_UUOPTION */ 1,
@@ -250,7 +250,7 @@ static char* knh_message_data[] = {
     /* KMSG_ESYNTAX */ 
         /* en */ "Syntax Error",
 #ifdef KNH_USING_I18N
-        /* ja */ "文法的な間違いがあります（ただしコンパイラのバグの可能性もあります）",
+        /* ja */ "文法的な間違いがあります（ただし、コンパイラのバグの可能性もあります）",
 #endif
         "ESYNTAX",
     
@@ -289,6 +289,13 @@ static char* knh_message_data[] = {
 #endif
         "WEOF",
     
+    /* KMSG_WINDENT */ 
+        /* en */ "Too short indent",
+#ifdef KNH_USING_I18N
+        /* ja */ "インデントがおかしいです",
+#endif
+        "WINDENT",
+    
     /* KMSG_WSEMICOLON */ 
         /* en */ "Add ';' at the end of the statement",
 #ifdef KNH_USING_I18N
@@ -576,19 +583,12 @@ static char* knh_message_data[] = {
 #endif
         "WLET",
     
-    /* KMSG_WEMPTYPEXPR */ 
-        /* en */ "Don't use the empty condition",
+    /* KMSG_WEMPTY */ 
+        /* en */ "Is it really empty?",
 #ifdef KNH_USING_I18N
-        /* ja */ "空っぽの条件式では困ります",
+        /* ja */ "空っぽです",
 #endif
-        "WEMPTYPEXPR",
-    
-    /* KMSG_WEMPTYARGS */ 
-        /* en */ "You will need some parameters",
-#ifdef KNH_USING_I18N
-        /* ja */ "引数を与えてください",
-#endif
-        "WEMPTYARGS",
+        "WEMPTY",
     
     /* KMSG_WTRUEFALSE */ 
         /* en */ NULL,
