@@ -97,7 +97,7 @@ static char* knh_token_array_tochar[] = {
     "instanceof",  /* Instanceof */ 
     "isa?",  /* Isa */ 
     "as",  /* As */ 
-    "in?",  /* In */ 
+    "in?",  /* Has */ 
     "===",  /* Is */ 
     "to?",  /* To */ 
     "==",  /* Eq */ 
@@ -218,7 +218,7 @@ static knh_methodn_t knh_token_array_tomethodn[] = {
     METHODN_opInstanceof,  /* Instanceof */ 
     METHODN_opIsa,  /* Isa */ 
     METHODN_opAs,  /* As */ 
-    METHODN_opIn,  /* In */ 
+    METHODN_opHas,  /* Has */ 
     METHODN_opIs,  /* Is */ 
     METHODN_opTo,  /* To */ 
     METHODN_opEq,  /* Eq */ 
@@ -339,7 +339,7 @@ static knh_bool_t knh_token_array_isEndOfStmt[] = {
     0,  /* Instanceof */ 
     0,  /* Isa */ 
     0,  /* As */ 
-    0,  /* In */ 
+    0,  /* Has */ 
     0,  /* Is */ 
     0,  /* To */ 
     0,  /* Eq */ 
@@ -580,7 +580,7 @@ Token *new_Token__parse(Ctx *ctx, knh_flag_t flag, knh_fileid_t fileid, knh_line
         if(ISB(t, "instanceof")) { tt = TT_INSTANCEOF; break; }
         if(ISB(t, "isa?")) { tt = TT_ISA; break; }
         if(ISB(t, "isa")) { tt = TT_ISA; break; }
-        if(ISB(t, "in?")) { tt = TT_IN; break; }
+        if(ISB(t, "in?")) { tt = TT_HAS; break; }
         if(ISB(t, "is?")) { tt = TT_IS; break; }
         if(ISB(t, "int")) { t = STEXT("Int!"); goto L_TAIL; }
     break;
@@ -1518,7 +1518,7 @@ new_StmtSTMT1(Ctx *ctx, knh_tokens_t *tc)
         break;
     case TT_AS:  /* As */
         break;
-    case TT_IN:  /* In */
+    case TT_HAS:  /* Has */
         break;
     case TT_IS:  /* Is */
         break;
