@@ -531,11 +531,10 @@ void knh_Object_free(Ctx *ctx, knh_Object_t *o)
 		}
 		break;
 
-	case CLASS_DictMap2 :
-	case CLASS_DictSet2 :
+	case CLASS_DictMap : case CLASS_DictSet :
 		{
-			knh_DictMap2_t *d = (knh_DictMap2_t*)o;
-			knh_array_traverse(ctx, d->list, knh_Object_RCsweep);
+			knh_DictMap_t *d = (knh_DictMap_t*)o;
+			knh_dict_traverse(ctx, d->_list, knh_Object_RCsweep);
 		}
 		break;
 
@@ -673,10 +672,10 @@ void knh_Object_traverse(Ctx *ctx, knh_Object_t *o, f_traverse ftr)
 		}
 		break;
 
-	case CLASS_DictMap2 : case CLASS_DictSet2 :
+	case CLASS_DictMap : case CLASS_DictSet :
 		{
-			knh_DictMap2_t *a = (knh_DictMap2_t*)o;
-			knh_array_traverse(ctx, a->list, ftr);
+			knh_DictMap_t *d = (knh_DictMap_t*)o;
+			knh_dict_traverse(ctx, d->_list, ftr);
 		}
 		break;
 
@@ -936,25 +935,25 @@ knh_System_t *new_System0(void)
 	knh_OutputStream_setEncoding(NULL, DP(o)->out, DP(o)->enc);
 	knh_OutputStream_setEncoding(NULL, DP(o)->err, DP(o)->enc);
 
-	KNH_INITv(DP(o)->props, new_DictMap(NULL, 64));
+	KNH_INITv(DP(o)->props, new_DictMap0(NULL, 64));
 
-	KNH_INITv(DP(o)->tConstDictMap, new_DictMap(NULL, 256));
+	KNH_INITv(DP(o)->tConstDictMap, new_DictMap0(NULL, 256));
 
 	KNH_INITv(DP(o)->tfieldnDictIdx, new_DictIdx(NULL, KNH_TFIELDN_SIZE * 2, 0));
 	KNH_INITv(DP(o)->tfileidDictIdx, new_DictIdx(NULL, 32, 0));
 	knh_DictIdx_add__fast(NULL, DP(o)->tfileidDictIdx, new_String__T(NULL, "(unknown)"));
-	KNH_INITv(DP(o)->tMethodFields, new_Array(NULL, CLASS_Any, KNH_TMETHODFIELD_SIZE * 2));
+	KNH_INITv(DP(o)->tMethodFields, new_Array0(NULL, KNH_TMETHODFIELD_SIZE * 2));
 
 	KNH_INITv(DP(o)->homeDir, new_String0(NULL, B(knh_format_homepath(buf,sizeof(buf))), NULL));
-	KNH_INITv(DP(o)->sysnsDictMap, new_DictMap(NULL, 16));
+	KNH_INITv(DP(o)->sysnsDictMap, new_DictMap0(NULL, 16));
 	KNH_INITv(DP(o)->funcDictSet, new_DictSet(NULL, 16));
 	KNH_INITv(DP(o)->drvapiDictSet, new_DictSet(NULL, 32));
 	KNH_INITv(DP(o)->makespecDictSet, new_DictSet(NULL, 32));
 
 	knh_DictMap_set(NULL, DP(o)->props, new_String__T(NULL, "konoha.version"), UP(new_String__T(NULL, KONOHA_VERSION)));
 	knh_DictMap_set(NULL, DP(o)->props, new_String__T(NULL, "konoha.encoding"), UP(DP(o)->enc));
-	knh_DictMap_set(NULL, DP(o)->props, new_String__T(NULL, "konoha.dir.home"), UP(DP(o)->homeDir));
-	knh_DictMap_set(NULL, DP(o)->props, new_String__T(NULL, "konoha.dir.package"),
+	knh_DictMap_set(NULL, DP(o)->props, new_String__T(NULL, "konoha.path.home"), UP(DP(o)->homeDir));
+	knh_DictMap_set(NULL, DP(o)->props, new_String__T(NULL, "konoha.path.package"),
 			UP(new_String0(NULL, B(knh_format_packagedir(buf, sizeof(buf),knh_String_tochar(DP(o)->homeDir))), NULL)));
 	knh_DictMap_set(NULL, DP(o)->props, new_String__T(NULL, "konoha.lang"),
 			UP(new_String0(NULL, B(knh_format_lang(buf, sizeof(buf))), NULL)));
